/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <zephyr/types.h>
#include <zephyr.h>
#include <device.h>
#include <soc.h>
#include <dk_buttons_and_leds.h>
#include <stdio.h>

#include <mbedtls/ecdh.h>
#include <mbedtls/entropy.h>
#include "mbedtls/ctr_drbg.h"

#include <logging/log.h>

#define LOG_MODULE_NAME ecdh_app
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

static mbedtls_entropy_context entropy;
static mbedtls_ctr_drbg_context ctr_drbg;
static mbedtls_ecdh_context initiator_ctx;
static mbedtls_ecdh_context responder_ctx;

void crypto_init(void)
{
    int ret;

    mbedtls_ecdh_init(&initiator_ctx);
	mbedtls_ecdh_init(&responder_ctx);
	mbedtls_ctr_drbg_init(&ctr_drbg);

	/*  Note: mbedtls_entropy_context argument must be zero initialized.
     *  See this DevZone post for more details:
     *  https://devzone.nordicsemi.com/f/nordic-q-a/58824/cc310-platform-mutex-init-error 
    */
	mbedtls_entropy_init(&entropy);

	ret = mbedtls_ctr_drbg_seed(&ctr_drbg, mbedtls_entropy_func, &entropy,
				    NULL, 0);

	if (ret != 0) {
		printk("Seeding failed");
	}
}

void crypto_ecdh_test(void)
{
    int err_code_initiator = -1;
	int err_code_responder = -1;
    
    err_code_initiator = mbedtls_ecp_group_load(&initiator_ctx.grp,
						    MBEDTLS_ECP_DP_SECP256R1);
	if (err_code_initiator != 0) {
        printk("mbedtls_ecp_group_load returned 0x%x", -err_code_initiator);
    }

	err_code_responder = mbedtls_ecp_group_load(&responder_ctx.grp,
						    MBEDTLS_ECP_DP_SECP256R1);
	if (err_code_responder != 0) {
        printk("mbedtls_ecp_group_load returned 0x%x", -err_code_responder);
    }

	err_code_initiator =
		mbedtls_ecdh_gen_public(&initiator_ctx.grp, &initiator_ctx.d,
					&initiator_ctx.Q,
					mbedtls_ctr_drbg_random, &ctr_drbg);
	if (err_code_initiator != 0) {
        printk("mbedtls_ecdh_gen_public returned 0x%x", -err_code_initiator);
    }

	err_code_responder =
		mbedtls_ecdh_gen_public(&responder_ctx.grp, &responder_ctx.d,
					&responder_ctx.Q,
					mbedtls_ctr_drbg_random, &ctr_drbg);
	if (err_code_initiator != 0) {
        printk("mbedtls_ecdh_gen_public returned 0x%x", -err_code_initiator);
    }

	err_code_initiator = mbedtls_ecdh_compute_shared(
		&initiator_ctx.grp, &initiator_ctx.z, &responder_ctx.Q,
		&initiator_ctx.d, mbedtls_ctr_drbg_random, &ctr_drbg);
    if (err_code_initiator != 0) {
        printk("mbedtls_ecdh_compute_shared returned 0x%x", -err_code_initiator);
    }
	err_code_responder = mbedtls_ecdh_compute_shared(
		&responder_ctx.grp, &responder_ctx.z, &initiator_ctx.Q,
		&responder_ctx.d, mbedtls_ctr_drbg_random, &ctr_drbg);
    if (err_code_responder != 0) {
        printk("mbedtls_ecdh_compute_shared returned 0x%x", -err_code_responder);
    }

    int ret = mbedtls_mpi_cmp_mpi(&initiator_ctx.z, &responder_ctx.z);
    if (ret != 0) {
	    printk("mbedtls_mpi_cmp_mpi returned 0x%x", -ret);
    } else {
	    printk("Shared secret is equal\n");
    }

    mbedtls_ecdh_free(&initiator_ctx);
    mbedtls_ecdh_free(&responder_ctx);
}

void main(void)
{
	printk("Start ECDH test with mbedtls\n");

    crypto_init();
    crypto_ecdh_test();

	printk("Test complete\n");
}